from collections.abc import Mapping as ABCMap
from collections.abc import MutableSequence as ABCMuSeq
from collections.abc import Set as ABCSet
from types import UnionType
from typing import Any, Callable, ForwardRef, Iterable, Literal, TypeVar, overload

from .base import (
    DirectPattern,
    DirectTypePattern,
    ForwardRefPattern,
    IterMode,
    MappingPattern,
    RegexPattern,
    SequencePattern,
    SwitchPattern,
    UnionPattern,
)
from .core import BasePattern, MatchMode
from .util import RawStr, TPattern

T1 = TypeVar("T1")
T2 = TypeVar("T2")

@overload
def parser(
    item: str, extra: str = "allow"
) -> (
    BasePattern[str, str, Literal[MatchMode.REGEX_MATCH]] | DirectPattern[str] | RegexPattern | UnionPattern
): ...
@overload
def parser(item: RawStr, extra: str = "allow") -> DirectPattern[str]: ...
@overload
def parser(item: TPattern, extra: str = "allow") -> RegexPattern: ...
@overload
def parser(
    item: type[ABCMap[T1, T2]], extra: str = "allow"
) -> MappingPattern[T1, T2, Literal[IterMode.ALL]]: ...
@overload
def parser(
    item: type[ABCMuSeq[T1]], extra: str = "allow"
) -> SequencePattern[list[T1], Literal[IterMode.ALL]]: ...
@overload
def parser(
    item: type[tuple[T1, ...]], extra: str = "allow"
) -> SequencePattern[tuple[T1, ...], Literal[IterMode.ALL]]: ...
@overload
def parser(
    item: type[ABCSet[T1]], extra: str = "allow"
) -> SequencePattern[set[T1], Literal[IterMode.ALL]]: ...
@overload
def parser(item: type[T1], extra: str = "allow") -> BasePattern[T1, Any, Any] | DirectTypePattern[T1]: ...
@overload
def parser(item: ABCMap[T1, T2], extra: str = "allow") -> SwitchPattern[T2, T1]: ...
@overload
def parser(item: UnionType, extra: str = "allow") -> UnionPattern[Any]: ...
@overload
def parser(item: Iterable[T1 | type[T1]], extra: str = "allow") -> UnionPattern[T1]: ...
@overload
def parser(item: ForwardRef, extra: str = "allow") -> ForwardRefPattern: ...
@overload
def parser(
    item: Callable[[T1], T2], extra: str = "allow"
) -> BasePattern[T2, T1, Literal[MatchMode.TYPE_CONVERT]]: ...
@overload
def parser(item: T1, extra: str = "allow") -> BasePattern[T1, T1, Literal[MatchMode.KEEP]]: ...



from __future__ import annotations

from functools import reduce
from typing import Any, Callable, Protocol, TypeVar, overload

from tarina import Empty

from .core import TMM, BasePattern, MatchMode, TInput

T = TypeVar("T")
T1 = TypeVar("T1")


def Index(
    pat: BasePattern[list[T], TInput, TMM],
    index: int,
) -> BasePattern[T, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return _match(input_)[index]

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}[{index}]"
    _new.refresh()
    return _new  # type: ignore


def Slice(
    pat: BasePattern[list[T], TInput, TMM],
    start: int | None = None,
    end: int | None = None,
    step: int = 1,
) -> BasePattern[list[T], TInput, TMM]:
    if start is None and end is None:
        return pat
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return _match(input_)[start:end:step]

    _new.match = match.__get__(_new)
    if start is not None and end is not None:
        _slice = f"{start}:{end}"
    elif start is not None:
        _slice = f"{start}:"
    else:
        _slice = f":{end}"
    if step != 1:
        _slice += f":{step}"
    _new.alias = f"{_new._repr}[{_slice}]"
    _new.refresh()
    return _new


def Map(
    pat: BasePattern[list[T], TInput, TMM],
    func: Callable[[T], T1],
    funcname: str | None = None,
) -> BasePattern[list[T1], TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return list(map(func, _match(input_)))

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.map({funcname or func.__name__})"
    _new.refresh()
    return _new  # type: ignore


def Filter(
    pat: BasePattern[list[T], TInput, TMM],
    func: Callable[[T], bool],
    funcname: str | None = None,
) -> BasePattern[list[T], TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return list(filter(func, _match(input_)))

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.filter({funcname or func.__name__})"
    _new.refresh()
    return _new


_T_contra = TypeVar("_T_contra", contravariant=True)
_T_co = TypeVar("_T_co", covariant=True)


class SupportsAdd(Protocol[_T_contra, _T_co]):
    def __add__(self, x: _T_contra, /) -> _T_co: ...


class SupportsRAdd(Protocol[_T_contra, _T_co]):
    def __radd__(self, x: _T_contra, /) -> _T_co: ...


class _SupportsSumWithNoDefaultGiven(SupportsAdd[Any, Any], SupportsRAdd[int, Any], Protocol): ...


_SupportsSumNoDefaultT = TypeVar("_SupportsSumNoDefaultT", bound=_SupportsSumWithNoDefaultGiven)


def Sum(
    pat: BasePattern[list[_SupportsSumNoDefaultT], TInput, TMM]
) -> BasePattern[_SupportsSumNoDefaultT, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return sum(_match(input_))

    _new.match = match.__get__(_new)
    _new.alias = f"sum({_new._repr})"
    _new.refresh()
    return _new  # type: ignore


@overload
def Reduce(
    pat: BasePattern[list[T], TInput, TMM],
    func: Callable[[T, T], T],
    initializer: None = ...,
    funcname: str | None = ...,
) -> BasePattern[T, TInput, TMM]: ...


@overload
def Reduce(
    pat: BasePattern[list[T], TInput, TMM],
    func: Callable[[T1, T], T1],
    initializer: T1,
    funcname: str | None = ...,
) -> BasePattern[T1, TInput, TMM]: ...


def Reduce(
    pat: BasePattern[list[T], TInput, TMM],
    func: Callable[[T, T], T] | Callable[[T1, T], T1],
    initializer: T1 | None = None,
    funcname: str | None = None,
) -> BasePattern[T, TInput, TMM] | BasePattern[T1, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return reduce(func, _match(input_), initializer) if initializer is not None else reduce(func, _match(input_))  # type: ignore

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.reduce({funcname or func.__name__})"
    _new.refresh()
    return _new  # type: ignore


def Join(
    pat: BasePattern[list[str], TInput, TMM],
    sep: str,
) -> BasePattern[str, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return sep.join(_match(input_))

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.join({sep!r})"
    _new.refresh()
    return _new  # type: ignore


def Upper(
    pat: BasePattern[str, TInput, TMM],
) -> BasePattern[str, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return _match(input_).upper()

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.upper()"
    _new.refresh()
    return _new  # type: ignore


def Lower(
    pat: BasePattern[str, TInput, TMM],
) -> BasePattern[str, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return _match(input_).lower()

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.lower()"
    _new.refresh()
    return _new  # type: ignore


def Dot(
    pat: BasePattern[Any, TInput, TMM],
    origin: type[T],
    key: str,
    default: T | None = None,
) -> BasePattern[T, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return getattr(_match(input_), key, default)

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.{key}"
    _new.refresh()
    _new.origin = origin
    return _new  # type: ignore


def GetItem(
    pat: BasePattern[Any, TInput, TMM],
    origin: type[T],
    key: str,
    default: T | None = None,
) -> BasePattern[T, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_) -> T:
        try:
            return _match(input_)[key]
        except Exception as e:
            if default is not None:
                return default
            raise e

    _new.match = match.__get__(_new)
    _new.alias = f"{_new._repr}.{key}"
    _new.refresh()
    _new.origin = origin
    return _new  # type: ignore


def Step(
    pat: BasePattern[T, TInput, TMM],
    func: Callable[[T], T1],
    *args,
    funcname: str | None = None,
    **kwargs,
) -> BasePattern[T1, TInput, TMM]:
    _new = pat.copy()
    _match = _new.match

    def match(self, input_):
        return func(_match(input_), *args, **kwargs)

    _new.match = match.__get__(_new)
    _new.alias = f"{funcname or func.__name__}({_new._repr})"
    _new.refresh()
    return _new  # type: ignore



from pathlib import Path
from typing import Any

from tarina import Empty as Empty  # noqa

from .base import ANY as ANY
from .base import AntiPattern as AntiPattern
from .base import AnyString as AnyString
from .base import BOOLEAN as BOOLEAN
from .base import BYTES as BYTES
from .base import DATETIME as DATETIME
from .base import DICT as DICT
from .base import DelimiterInt as DelimiterInt
from .base import DirectPattern as DirectPattern
from .base import DirectTypePattern as DirectTypePattern
from .base import EMAIL as EMAIL
from .base import FLOAT as FLOAT
from .base import HEX as HEX
from .base import HEX_COLOR as HEX_COLOR
from .base import INTEGER as INTEGER
from .base import IP as IP
from .base import IterMode as IterMode
from .base import LIST as LIST
from .base import MappingPattern as MappingPattern
from .base import NONE as NONE
from .base import NUMBER as NUMBER
from .base import PATH as PATH
from .base import PathFile as PathFile
from .base import RegexPattern as RegexPattern
from .base import SET as SET
from .base import STRING as STRING
from .base import SequencePattern as SequencePattern
from .base import SwitchPattern as SwitchPattern
from .base import TUPLE as TUPLE
from .base import URL as URL
from .base import UnionPattern as UnionPattern
from .base import WIDE_BOOLEAN as WIDE_BOOLEAN
from .base import combine as combine
from .context import Patterns as Patterns
from .context import all_patterns as all_patterns
from .context import create_local_patterns as create_local_patterns
from .context import global_patterns
from .context import local_patterns as local_patterns
from .context import reset_local_patterns as reset_local_patterns
from .context import switch_local_patterns as switch_local_patterns
from .core import BasePattern as BasePattern
from .core import MatchMode as MatchMode
from .core import ValidateResult as ValidateResult
from .exception import MatchFailed as MatchFailed
from .func import Dot as Dot
from .func import Filter as Filter
from .func import GetItem as GetItem
from .func import Index as Index
from .func import Join as Join
from .func import Lower as Lower
from .func import Map as Map
from .func import Reduce as Reduce
from .func import Slice as Slice
from .func import Step as Step
from .func import Sum as Sum
from .func import Upper as Upper
from .main import parser as parser
from .util import RawStr as RawStr
from .util import TPattern as TPattern

type_parser = parser

global_patterns().update(
    {
        Any: ANY,
        Ellipsis: ANY,
        object: ANY,
        "any": ANY,
        "any_str": AnyString,
        "email": EMAIL,
        "color": HEX_COLOR,
        "hex": HEX,
        "ip": IP,
        "url": URL,
        "...": ANY,
        "number": NUMBER,
        list: LIST,
        tuple: TUPLE,
        set: SET,
        dict: DICT,
        Path: PATH,
        "file": PathFile,
    }
)


global_patterns().sets([BYTES, STRING, INTEGER, FLOAT, BOOLEAN, DATETIME])



from __future__ import annotations

import dataclasses
import sys
from typing import TYPE_CHECKING, List, Pattern, Union

from .i18n import lang as lang

if sys.version_info >= (3, 9):  # pragma: no cover
    from types import GenericAlias as CGenericAlias  # noqa
else:  # pragma: no cover
    CGenericAlias: type = type(List[int])  # noqa

if sys.version_info >= (3, 10):  # pragma: no cover
    from types import UnionType as CUnionType  # noqa
else:  # pragma: no cover
    CUnionType: type = type(Union[int, str])  # noqa

if sys.version_info >= (3, 11):  # pragma: no cover
    from re._compiler import compile as re_compile  # noqa
else:  # pragma: no cover
    from sre_compile import compile as re_compile  # noqa

if TYPE_CHECKING:
    TPattern = Pattern[str]
else:
    TPattern: type[Pattern[str]] = type(re_compile("", 0))
GenericAlias: type = type(List[int])
UnionType: type = type(Union[int, str])


@dataclasses.dataclass
class RawStr:
    value: str



from __future__ import annotations

from copy import deepcopy
from enum import Enum, IntEnum
import re
from typing import TYPE_CHECKING, Any, Callable, Generic, Literal, TypeVar
from typing_extensions import Self, get_args, get_origin

from tarina import Empty, generic_isinstance
from tarina.lang import lang

from .exception import MatchFailed


class MatchMode(IntEnum):
    """参数表达式匹配模式"""

    VALUE_OPERATE = 4
    """传入值进行操作"""
    REGEX_CONVERT = 3
    """正则匹配并转换"""
    TYPE_CONVERT = 2
    """传入值直接转换"""
    REGEX_MATCH = 1
    """正则匹配"""
    KEEP = 0
    """保持传入值"""


class ResultFlag(str, Enum):
    """参数表达式验证结果标识符"""

    VALID = "valid"
    "验证通过"
    ERROR = "error"
    "验证失败"
    DEFAULT = "default"
    "默认值"


T = TypeVar("T")
TInput = TypeVar("TInput")
TOrigin = TypeVar("TOrigin")
TVOrigin = TypeVar("TVOrigin")
TDefault = TypeVar("TDefault")
TVRF = TypeVar("TVRF", bound=ResultFlag)
TMM = TypeVar("TMM", bound=MatchMode)


class ValidateResult(Generic[TVOrigin, TVRF]):
    """参数表达式验证结果"""

    def __init__(
        self,
        value: TVOrigin | type[Empty] = Empty,
        error: Exception | type[Empty] = Empty,
        flag: TVRF = ResultFlag.VALID,
    ):
        self._value = value
        self._error = error
        self.flag = flag

    __slots__ = ("_value", "_error", "flag")

    def value(self) -> TVOrigin:
        if self.flag == ResultFlag.ERROR or self._value == Empty:
            raise RuntimeError("cannot access value")
        return self._value  # type: ignore

    def error(self) -> Exception | None:
        if self.flag == ResultFlag.ERROR and self._error != Empty:
            assert isinstance(self._error, Exception)
            return self._error

    @property
    def success(self) -> bool:
        return self.flag == ResultFlag.VALID

    @property
    def failed(self) -> bool:
        return self.flag == ResultFlag.ERROR

    @property
    def or_default(self) -> bool:
        return self.flag == ResultFlag.DEFAULT

    def step(
        self, other: type[T] | Callable[[TVOrigin], T] | Any | BasePattern[T, TVOrigin, MatchMode]
    ) -> T | Self | ValidateResult[T, TVRF]:
        if other is bool:
            return self.success  # type: ignore
        if callable(other) and self.success:
            return other(self.value())  # type: ignore
        return other.validate(self.value()) if isinstance(other, BasePattern) else self  # type: ignore

    def __rshift__(
        self, other: type[T] | Callable[[TVOrigin], T] | Any
    ) -> T | Self | ValidateResult[T, TVRF]:
        return self.step(other)  # type: ignore

    def __bool__(self):
        return self.flag != ResultFlag.ERROR

    def __repr__(self):
        if self.flag == ResultFlag.VALID:
            return f"ValidateResult(value={self._value!r})"
        if self.flag == ResultFlag.ERROR:
            return f"ValidateResult(error={self._error!r})"
        return f"ValidateResult(default={self._value!r})"


def _keep_any(self: BasePattern[Any, Any, Literal[MatchMode.KEEP]], input_: Any) -> Any:
    return input_


def _keep_no_previous(self: BasePattern[Any, Any, Literal[MatchMode.KEEP]], input_: Any) -> Any:
    if not self.accept(input_):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    return input_


def _keep_previous(self: BasePattern[Any, Any, Literal[MatchMode.KEEP]], input_: Any) -> Any:
    if TYPE_CHECKING:
        assert self.previous
    if self.accept(input_):
        input_ = self.previous.match(input_)
    elif not self.accept(input_ := self.previous.match(input_)):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    return input_


def select_keep_match(self: BasePattern[Any, Any, Literal[MatchMode.KEEP]]):
    if self._accepts or self._pattern_accepts:
        return _keep_previous if self.previous else _keep_no_previous
    return _keep_any


def _regex_match_no_previous(self: BasePattern[str, str, Literal[MatchMode.REGEX_MATCH]], input_: Any) -> str:
    if not isinstance(input_, str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        return mat[0]
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def _regex_match_type(self: BasePattern[str, str, Literal[MatchMode.REGEX_MATCH]], input_: Any) -> str:
    if TYPE_CHECKING:
        assert self.previous
    if not isinstance(input_, str) and not isinstance(
        input_ := self.previous.match(input_), str
    ):  # pragma: no cover
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        return mat[0]
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def _regex_match_value(self: BasePattern[str, str, Literal[MatchMode.REGEX_MATCH]], input_: Any) -> str:
    if TYPE_CHECKING:
        assert self.previous
    if not isinstance(input_, str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    input_ = self.previous.match(input_)
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        return mat[0]
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def select_regex_match(self: BasePattern[Any, Any, Literal[MatchMode.REGEX_MATCH]]):
    if self.previous:
        if self.previous.mode == MatchMode.TYPE_CONVERT:
            return _regex_match_type
        return _regex_match_value
    return _regex_match_no_previous


def _regex_convert_no_previous_any(
    self: BasePattern[TOrigin, str | TOrigin, Literal[MatchMode.REGEX_CONVERT]], input_: Any
) -> TOrigin:
    if not isinstance(input_, str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        if (res := self.converter(self, mat)) is not None:
            return res
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def _regex_convert_no_previous_other(
    self: BasePattern[TOrigin, str | TOrigin, Literal[MatchMode.REGEX_CONVERT]], input_: Any
) -> TOrigin:
    if generic_isinstance(input_, self.origin):
        return input_  # type: ignore
    if not isinstance(input_, str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        if (res := self.converter(self, mat)) is not None:
            return res
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def _regex_convert_any_type(
    self: BasePattern[TOrigin, str | TOrigin, Literal[MatchMode.REGEX_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if not isinstance(input_, str) and not isinstance(input_ := self.previous.match(input_), str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        if (res := self.converter(self, mat)) is not None:
            return res
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def _regex_convert_any_value(
    self: BasePattern[TOrigin, str | TOrigin, Literal[MatchMode.REGEX_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if not isinstance(input_, str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    input_ = self.previous.match(input_)
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        if (res := self.converter(self, mat)) is not None:
            return res
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def _regex_convert_value(
    self: BasePattern[TOrigin, str | TOrigin, Literal[MatchMode.REGEX_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    input_ = self.previous.match(input_)
    if generic_isinstance(input_, self.origin):
        return input_  # type: ignore
    if not isinstance(input_, str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        if (res := self.converter(self, mat)) is not None:
            return res
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def _regex_convert_type(
    self: BasePattern[TOrigin, str | TOrigin, Literal[MatchMode.REGEX_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if generic_isinstance(input_, self.origin):
        return input_  # type: ignore
    if not isinstance(input_, str) and not isinstance(input_ := self.previous.match(input_), str):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if mat := (self.regex_pattern.match(input_) or self.regex_pattern.search(input_)):
        if (res := self.converter(self, mat)) is not None:
            return res
    raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected=self.alias))


def select_regex_convert(self: BasePattern[Any, Any, Literal[MatchMode.REGEX_CONVERT]]):
    if self.origin is Any or self.origin is str:
        if not self.previous:
            return _regex_convert_no_previous_any
        return (
            _regex_convert_any_value
            if self.previous.mode == MatchMode.VALUE_OPERATE
            else _regex_convert_any_type
        )
    if not self.previous:
        return _regex_convert_no_previous_other
    if self.previous.mode == MatchMode.VALUE_OPERATE:
        return _regex_convert_value
    return _regex_convert_type


def _type_convert_no_previous_no_accepts_any(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_no_previous_no_accepts_other(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if generic_isinstance(input_, self.origin):
        return input_
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_no_previous_accepts_any(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if not self.accept(input_):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_no_previous_accepts_other(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if not self.accept(input_):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if generic_isinstance(input_, self.origin):
        return input_
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_type_no_accepts_any(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if (res := self.converter(self, input_)) is None and (
        res := self.converter(self, self.previous.match(input_))
    ) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_type_no_accepts_other(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if generic_isinstance(input_, self.origin):
        return input_
    if (res := self.converter(self, input_)) is None and (
        res := self.converter(self, self.previous.match(input_))
    ) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_type_accepts_any(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if not self.accept(input_) and not self.accept(input_ := self.previous.match(input_)):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if (res := self.converter(self, input_)) is None and (
        res := self.converter(self, self.previous.match(input_))
    ) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_type_accepts_other(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if not self.accept(input_) and not self.accept(input_ := self.previous.match(input_)):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if generic_isinstance(input_, self.origin):
        return input_
    if (res := self.converter(self, input_)) is None and (
        res := self.converter(self, self.previous.match(input_))
    ) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_value_no_accepts_any(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    input_ = self.previous.match(input_)
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_value_no_accepts_other(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    input_ = self.previous.match(input_)
    if generic_isinstance(input_, self.origin):
        return input_
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_value_accepts_any(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if self.accept(input_):
        input_ = self.previous.match(input_)
    elif not self.accept(input_ := self.previous.match(input_)):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _type_convert_value_accepts_other(
    self: BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if self.accept(input_):
        input_ = self.previous.match(input_)
    elif not self.accept(input_ := self.previous.match(input_)):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if generic_isinstance(input_, self.origin):
        return input_
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def select_type_convert(self: BasePattern[Any, Any, Literal[MatchMode.TYPE_CONVERT]]):
    if self._accepts or self._pattern_accepts:
        if self.origin is Any:
            if not self.previous:
                return _type_convert_no_previous_accepts_any
            return _type_convert_value_accepts_any if self.previous else _type_convert_type_accepts_any
        if not self.previous:
            return _type_convert_no_previous_accepts_other
        return _type_convert_value_accepts_other if self.previous else _type_convert_type_accepts_other
    if self.origin is Any:
        if not self.previous:
            return _type_convert_no_previous_no_accepts_any
        return _type_convert_value_no_accepts_any if self.previous else _type_convert_type_no_accepts_any
    if not self.previous:
        return _type_convert_no_previous_no_accepts_other
    return _type_convert_value_no_accepts_other if self.previous else _type_convert_type_no_accepts_other


def _value_operate_no_previous(
    self: BasePattern[TOrigin, TOrigin, Literal[MatchMode.VALUE_OPERATE]], input_: Any
) -> TOrigin:
    if not generic_isinstance(input_, self.origin):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _value_operate_type(
    self: BasePattern[TOrigin, TOrigin, Literal[MatchMode.VALUE_OPERATE]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if not generic_isinstance(input_, self.origin) and not generic_isinstance(
        input_ := self.previous.match(input_), self.origin
    ):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def _value_operate_value(
    self: BasePattern[TOrigin, TOrigin, Literal[MatchMode.VALUE_OPERATE]], input_: Any
) -> TOrigin:
    if TYPE_CHECKING:
        assert self.previous
    if not generic_isinstance(input_, self.origin):
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected=self.alias
            )
        )
    input_ = self.previous.match(input_)
    if (res := self.converter(self, input_)) is None:
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
        )
    return res  # type: ignore


def select_value_operate(self: BasePattern[Any, Any, Literal[MatchMode.VALUE_OPERATE]]):
    if self.previous:
        if self.previous.mode == MatchMode.TYPE_CONVERT:
            return _value_operate_type
        return _value_operate_value
    return _value_operate_no_previous


_MATCHES = {
    MatchMode.KEEP: select_keep_match,
    MatchMode.REGEX_MATCH: select_regex_match,
    MatchMode.REGEX_CONVERT: select_regex_convert,
    MatchMode.TYPE_CONVERT: select_type_convert,
    MatchMode.VALUE_OPERATE: select_value_operate,
}


class BasePattern(Generic[TOrigin, TInput, TMM]):
    """对参数类型值的包装"""

    __slots__ = (
        "regex_pattern",
        "pattern",
        "mode",
        "converter",
        "origin",
        "alias",
        "previous",
        "validators",
        "match",
        "accept",
        "_accepts",
        "_pattern_accepts",
        "_hash",
        "_repr",
    )

    def __new__(cls, *args, **kwargs):
        cls.__eq__ = cls.__calc_eq__
        return super().__new__(cls)

    def __init__(
        self,
        pattern: str | None = None,
        mode: TMM | None = None,
        origin: type[TOrigin] | None = None,
        converter: Callable[[BasePattern, Any], TOrigin | None] | None = None,
        alias: str | None = None,
        previous: BasePattern | None = None,
        accepts: Any = None,
        addition_accepts: BasePattern | None = None,
        validators: list[Callable[[TOrigin], bool]] | None = None,
    ):
        """
        初始化参数匹配表达式
        """
        if pattern is None:
            _origin = origin or Any
            self.mode = MatchMode(mode or MatchMode.KEEP)
            self.pattern = ""
            self.regex_pattern = re.compile("")
        else:
            if pattern.startswith("^") or pattern.endswith("$"):
                raise ValueError(
                    lang.require("nepattern", "pattern_head_or_tail_error").format(target=pattern)
                )
            self.pattern = pattern
            self.regex_pattern = re.compile(f"^{pattern}$")
            self.mode = MatchMode(mode or MatchMode.REGEX_MATCH)
            _origin = origin or str
        self.origin: type[TOrigin] = _origin  # type: ignore
        self.alias = alias
        self.previous = previous
        if TYPE_CHECKING:
            assert self.origin is not None
        if self.mode == MatchMode.TYPE_CONVERT:
            if not converter and (not origin or origin is Any):
                raise ValueError(origin)
            self.converter = converter or (lambda _, x: (get_origin(self.origin) or self.origin)(x))  # type: ignore
        elif self.mode == MatchMode.VALUE_OPERATE:
            if not converter:
                raise ValueError(converter)
            self.converter = converter
        else:
            self.converter = converter or (lambda _, x: eval(x[0]))
        self.validators = validators or []
        if accepts is Any or not accepts:
            _accepts = Any
            self._accepts = ()
        else:
            _accepts = get_args(accepts) or accepts
            self._accepts = get_args(accepts) or (accepts,)
        self._pattern_accepts = addition_accepts
        self._repr = self.__calc_repr__()
        self._hash = self.__calc_hash__()

        if not addition_accepts:
            self.accept = (lambda x: True) if _accepts is Any else (lambda _: generic_isinstance(_, _accepts))
        elif _accepts is Any:
            self.accept = lambda _: addition_accepts.validate(_).flag == "valid"
        else:
            self.accept = lambda _: (
                generic_isinstance(_, _accepts) or addition_accepts.validate(_).flag == "valid"
            )
        if not hasattr(self, "match"):
            self.match = _MATCHES[self.mode](self).__get__(self)  # type: ignore

    def refresh(self):  # pragma: no cover
        self._repr = self.__calc_repr__()
        self._hash = self.__calc_hash__()

    def __calc_hash__(self):
        return hash(
            (self._repr, self.origin, self.mode, self.alias, self.previous, self._accepts, self.pattern)
        )

    def __calc_repr__(self):
        if self.mode == MatchMode.KEEP:
            if self.alias:
                return self.alias
            return (
                "Any"
                if not self._accepts and not self._pattern_accepts
                else "|".join([x.__name__ for x in self._accepts] + [self._pattern_accepts.__repr__()])
            )

        if not self.alias:
            name = getattr(self.origin, "__name__", str(self.origin))
            if self.mode == MatchMode.REGEX_MATCH:
                text = self.pattern
            elif self.mode == MatchMode.REGEX_CONVERT or (not self._accepts and not self._pattern_accepts):
                text = name
            else:
                text = (
                    "|".join([x.__name__ for x in self._accepts] + [self._pattern_accepts.__repr__()])
                    + f" -> {name}"
                )
        else:
            text = self.alias
        return (
            f"{f'{self.previous.__repr__()} -> ' if self.previous and id(self.previous) != id(self) else ''}"
            f"{text}"
        )

    def __repr__(self):
        return self._repr

    def __str__(self):
        return self._repr

    def __hash__(self):
        return self._hash

    def __calc_eq__(self, other):
        return isinstance(other, self.__class__) and self._hash == other._hash

    @staticmethod
    def of(unit: type[TOrigin]):
        """提供 Type[DataUnit] 类型的构造方法"""
        from .base import DirectTypePattern

        return DirectTypePattern(unit)

    @staticmethod
    def on(obj: TOrigin):
        """提供 DataUnit 类型的构造方法"""
        from .base import DirectPattern

        return DirectPattern(obj)

    @staticmethod
    def to(content: Any):
        """便捷的使用 type_parser 的方法"""
        from .main import parser

        res = parser(content, "allow")
        return res if isinstance(res, BasePattern) else parser(Any)  # type: ignore

    def validate(self, input_: Any, default: TDefault | Empty = Empty) -> ValidateResult[TOrigin | TDefault, ResultFlag]:  # type: ignore
        """
        对传入的值进行正向验证，返回可能的匹配与转化结果。

        若传入默认值，验证失败会返回默认值
        """
        try:
            res = self.match(input_)
            if self.validators and not all(i(res) for i in self.validators):
                raise MatchFailed(lang.require("nepattern", "validate_error").format(target=input_))
            return ValidateResult(value=res, flag=ResultFlag.VALID)
        except Exception as e:
            if default is Empty:
                return ValidateResult(error=e, flag=ResultFlag.ERROR)
            return ValidateResult(default, flag=ResultFlag.DEFAULT)  # type: ignore

    def copy(self) -> Self:
        return deepcopy(self)

    def __rrshift__(self, other):
        return self.validate(other)

    def __rmatmul__(self, other) -> Self:  # pragma: no cover
        if isinstance(other, str):
            self.alias = other
        return self

    def __matmul__(self, other) -> Self:  # pragma: no cover
        if isinstance(other, str):
            self.alias = other
        return self

    def __or__(self, other):
        from .base import UnionPattern

        if isinstance(other, BasePattern):
            return UnionPattern([self, other])  # type: ignore
        raise TypeError(  # pragma: no cover
            f"unsupported operand type(s) for |: 'BasePattern' and '{other.__class__.__name__}'"
        )



from __future__ import annotations

from datetime import datetime
from enum import Enum
from pathlib import Path
import re
import sys
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Dict,
    ForwardRef,
    Generic,
    Iterable,
    Literal,
    Match,
    TypeVar,
    Union,
    cast,
    final,
    overload,
)

from tarina import DateParser, Empty, lang

from .core import _MATCHES, TMM, BasePattern, MatchMode, ResultFlag, ValidateResult
from .exception import MatchFailed
from .util import TPattern

TOrigin = TypeVar("TOrigin")
TDefault = TypeVar("TDefault")
_T = TypeVar("_T")
_T1 = TypeVar("_T1")


class DirectPattern(BasePattern[TOrigin, TOrigin, Literal[MatchMode.KEEP]]):
    """直接判断"""

    def __init__(self, target: TOrigin, alias: str | None = None):
        self.target = target
        super().__init__(mode=MatchMode.KEEP, origin=type(target), alias=alias or str(target))

    def match(self, input_: Any):
        if input_ != self.target:
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected=self.target)
            )
        return input_

    @overload
    def validate(self, input_: TOrigin) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...

    @overload
    def validate(self, input_: _T) -> ValidateResult[_T, Literal[ResultFlag.ERROR]]: ...

    @overload
    def validate(
        self, input_: TOrigin, default: Any
    ) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...

    @overload
    def validate(
        self, input_: Any, default: TDefault
    ) -> ValidateResult[TDefault, Literal[ResultFlag.DEFAULT]]: ...

    def validate(self, input_: Any, default: Union[TDefault, Empty] = Empty) -> ValidateResult[TOrigin | TDefault, ResultFlag]:  # type: ignore
        try:
            return ValidateResult(self.match(input_), flag=ResultFlag.VALID)
        except MatchFailed as e:
            if default is Empty:
                return ValidateResult(error=e, flag=ResultFlag.ERROR)
            return ValidateResult(default, flag=ResultFlag.DEFAULT)  # type: ignore

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, DirectPattern) and self.target == other.target

    def copy(self):
        return DirectPattern(self.target, self.alias)


class DirectTypePattern(BasePattern[TOrigin, TOrigin, Literal[MatchMode.KEEP]]):
    """直接类型判断"""

    def __init__(self, target: type[TOrigin], alias: str | None = None):
        self.target = target
        super().__init__(mode=MatchMode.KEEP, origin=target, alias=alias or target.__name__)

    def match(self, input_: Any):
        if not isinstance(input_, self.target):
            raise MatchFailed(
                lang.require("nepattern", "type_error").format(
                    type=input_.__class__, target=input_, expected=self.target
                )
            )
        return input_

    @overload
    def validate(self, input_: TOrigin) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...

    @overload
    def validate(self, input_: _T) -> ValidateResult[_T, Literal[ResultFlag.ERROR]]: ...

    @overload
    def validate(
        self, input_: TOrigin, default: Any
    ) -> ValidateResult[TOrigin, Literal[ResultFlag.VALID]]: ...

    @overload
    def validate(
        self, input_: Any, default: TDefault
    ) -> ValidateResult[TDefault, Literal[ResultFlag.DEFAULT]]: ...

    def validate(self, input_: Any, default: Union[TDefault, Empty] = Empty) -> ValidateResult[TOrigin | TDefault, ResultFlag]:  # type: ignore
        try:
            return ValidateResult(self.match(input_), flag=ResultFlag.VALID)
        except MatchFailed as e:
            if default is Empty:
                return ValidateResult(error=e, flag=ResultFlag.ERROR)
            return ValidateResult(default, flag=ResultFlag.DEFAULT)  # type: ignore

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, DirectTypePattern) and self.target == other.target

    def copy(self):
        return DirectTypePattern(self.target, self.alias)


class RegexPattern(BasePattern[Match[str], str, Literal[MatchMode.REGEX_MATCH]]):
    """针对正则的特化匹配，支持正则组"""

    def __init__(self, pattern: str | TPattern, alias: str | None = None):
        super().__init__("", origin=Match[str], alias=alias or "regex[:group]")  # type: ignore
        self.regex_pattern = re.compile(pattern)
        self.pattern = self.regex_pattern.pattern

    def match(self, input_: Any) -> Match[str]:
        if not isinstance(input_, str):
            raise MatchFailed(
                lang.require("nepattern", "type_error").format(
                    type=input_.__class__, target=input_, expected="str"
                )
            )
        if mat := self.regex_pattern.match(input_):
            return mat
        raise MatchFailed(
            lang.require("nepattern", "content_error").format(target=input_, expected=self.pattern)
        )

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, RegexPattern) and self.pattern == other.pattern

    def copy(self):
        return RegexPattern(self.pattern, self.alias)


class UnionPattern(BasePattern[Any, _T, Literal[MatchMode.KEEP]]):
    """多类型参数的匹配"""

    optional: bool
    for_validate: list[BasePattern]
    for_equal: list[str | object]

    __slots__ = ("base", "optional", "for_validate", "for_equal")

    def __init__(self, base: Iterable[BasePattern[Any, _T, Any] | _T]):
        self.base = list(base)
        self.optional = False
        self.for_validate = []
        self.for_equal = []

        for arg in self.base:
            if arg == NONE:
                self.optional = True
                self.for_equal.append(None)
            elif isinstance(arg, BasePattern):
                self.for_validate.append(arg)
            else:
                self.for_equal.append(arg)
        alias_content = "|".join([repr(a) for a in self.for_validate] + [repr(a) for a in self.for_equal])
        super().__init__(mode=MatchMode.KEEP, origin=Any, alias=alias_content)

    def match(self, input_: Any):
        if not input_:
            input_ = None
        if input_ not in self.for_equal:
            for pat in self.for_validate:
                if (res := pat.validate(input_)).success:
                    return res.value()
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected=self.alias)
            )
        return input_

    @classmethod
    def _(cls, *types: type[_T1]) -> UnionPattern[_T1]:
        from .main import parser

        return cls([parser(i) for i in types])  # type: ignore

    def __calc_repr__(self):
        return "|".join(repr(a) for a in (*self.for_validate, *self.for_equal))

    def __or__(self, other: BasePattern[Any, _T1, Any]) -> UnionPattern[Union[_T, _T1]]:
        return UnionPattern([*self.base, other])  # type: ignore

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, UnionPattern) and self.base == other.base


TSeq = TypeVar("TSeq", list, tuple, set)


class IterMode(str, Enum):
    PRE = "pre"
    SUF = "suf"
    ALL = "all"


TIterMode = TypeVar("TIterMode", bound=IterMode)


class SequencePattern(
    BasePattern[TSeq, Union[str, TSeq], Literal[MatchMode.REGEX_CONVERT]], Generic[TSeq, TIterMode]
):
    """匹配列表或者元组或者集合"""

    base: BasePattern
    itermode: TIterMode

    def __init__(self, form: type[TSeq], base: BasePattern, mode: TIterMode = IterMode.ALL):
        self.base = base
        self.itermode = mode
        if form is list:
            super().__init__(r"\[(.+?)\]", MatchMode.REGEX_CONVERT, form, alias=f"list[{base}]")
        elif form is tuple:
            super().__init__(r"\((.+?)\)", MatchMode.REGEX_CONVERT, form, alias=f"tuple[{base}]")
        elif form is set:
            super().__init__(r"\{(.+?)\}", MatchMode.REGEX_CONVERT, form, alias=f"set[{base}]")
        else:
            raise ValueError(lang.require("nepattern", "sequence_form_error").format(target=str(form)))
        self.converter = lambda _, x: x[1]  # type: ignore
        self._match = _MATCHES[MatchMode.REGEX_CONVERT](self)

    def match(self, input_: Any):
        _res = self._match(self, input_)  # type: ignore
        _max = 0
        success: list[tuple[int, Any]] = []
        fail: list[tuple[int, MatchFailed]] = []
        for _max, s in enumerate(re.split(r"\s*,\s*", _res) if isinstance(_res, str) else _res):
            try:
                success.append((_max, self.base.match(s)))
            except MatchFailed:
                fail.append((_max, MatchFailed(f"{s} is not matched with {self.base}")))

        if (
            (self.itermode == IterMode.ALL and fail)
            or (self.itermode == IterMode.PRE and fail and fail[0][0] == 0)
            or (self.itermode == IterMode.SUF and fail and fail[-1][0] == _max)
        ):
            raise fail[0][1]
        if self.itermode == IterMode.PRE and fail:
            return self.origin(i[1] for i in success if i[0] < fail[0][0])
        if self.itermode == IterMode.SUF and fail:
            return self.origin(i[1] for i in success if i[0] > fail[-1][0])
        return self.origin(i[1] for i in success)

    def __calc_repr__(self):
        return f"{self.origin.__name__}[{self.base}]"


TKey = TypeVar("TKey")
TVal = TypeVar("TVal")


class MappingPattern(
    BasePattern[Dict[TKey, TVal], Union[str, Dict[TKey, TVal]], Literal[MatchMode.REGEX_CONVERT]],
    Generic[TKey, TVal, TIterMode],
):
    """匹配字典或者映射表"""

    key: BasePattern[TKey, Any, Any]
    value: BasePattern[TVal, Any, Any]
    itermode: TIterMode

    def __init__(
        self,
        arg_key: BasePattern[TKey, Any, Any],
        arg_value: BasePattern[TVal, Any, Any],
        mode: TIterMode = IterMode.ALL,
    ):
        self.key = arg_key
        self.value = arg_value
        self.itermode = mode
        super().__init__(
            r"\{(.+?)\}",
            MatchMode.REGEX_CONVERT,
            dict,
            alias=f"dict[{self.key}, {self.value}]",
        )
        self.converter = lambda _, x: x[1]
        self._match = _MATCHES[MatchMode.REGEX_CONVERT](self)

    def match(self, input_: str | dict):
        _res = self._match(self, input_)  # type: ignore
        success: list[tuple[int, Any, Any]] = []
        fail: list[tuple[int, MatchFailed]] = []
        _max = 0

        def _generator_items(res: str | dict):
            if isinstance(res, dict):
                yield from res.items()
                return
            for m in re.split(r"\s*,\s*", res):
                yield re.split(r"\s*[:=]\s*", m)

        for _max, item in enumerate(_generator_items(_res)):
            k, v = item
            try:
                success.append((_max, self.key.match(k), self.value.match(v)))
            except MatchFailed:
                fail.append(
                    (
                        _max,
                        MatchFailed(f"{k}: {v} is not matched with {self.key}: {self.value}"),
                    )
                )
        if (
            (self.itermode == IterMode.ALL and fail)
            or (self.itermode == IterMode.PRE and fail and fail[0][0] == 0)
            or (self.itermode == IterMode.SUF and fail and fail[-1][0] == _max)
        ):
            raise fail[0][1]
        if self.itermode == IterMode.PRE and fail:
            return {i[1]: i[2] for i in success if i[0] < fail[0][0]}
        if self.itermode == IterMode.SUF and fail:
            return {i[1]: i[2] for i in success if i[0] > fail[-1][0]}
        return {i[1]: i[2] for i in success}

    def __calc_repr__(self):
        return f"dict[{self.key.origin.__name__}, {self.value}]"


_TCase = TypeVar("_TCase")
_TSwtich = TypeVar("_TSwtich")


class SwitchPattern(BasePattern[_TCase, _TSwtich, Literal[MatchMode.TYPE_CONVERT]]):
    switch: dict[_TSwtich | ellipsis, _TCase]

    __slots__ = ("switch",)

    def __init__(self, data: dict[_TSwtich | ellipsis, _TCase]):
        self.switch = data
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=type(list(data.values())[0]))

    def __calc_repr__(self):
        return "|".join(f"{k}" for k in self.switch if k != Ellipsis)

    def match(self, input_: Any) -> _TCase:
        try:
            return self.switch[input_]
        except KeyError as e:
            if Ellipsis in self.switch:
                return self.switch[...]
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected=self._repr)
            ) from e

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, SwitchPattern) and self.switch == other.switch


class ForwardRefPattern(BasePattern[Any, Any, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self, ref: ForwardRef):
        self.ref = ref
        super().__init__(
            mode=MatchMode.TYPE_CONVERT, origin=Any, converter=lambda _, x: eval(x), alias=ref.__forward_arg__
        )

    def match(self, input_: Any):
        if isinstance(input_, str) and input_ == self.ref.__forward_arg__:
            return input_
        _main = sys.modules["__main__"]
        if sys.version_info < (3, 9):  # pragma: no cover
            origin = self.ref._evaluate(_main.__dict__, _main.__dict__)
        else:  # pragma: no cover
            origin = self.ref._evaluate(_main.__dict__, _main.__dict__, frozenset())  # type: ignore
        if not isinstance(input_, origin):  # type: ignore
            raise MatchFailed(
                lang.require("nepattern", "type_error").format(
                    type=input_.__class__, target=input_, expected=self.ref.__forward_arg__
                )
            )
        return input_

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, ForwardRefPattern) and self.ref == other.ref


class AntiPattern(BasePattern[TOrigin, Any, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self, pattern: BasePattern[TOrigin, Any, Any]):
        self.base: BasePattern[TOrigin, Any, Any] = pattern
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=pattern.origin, alias=f"!{pattern}")

    @overload
    def validate(self, input_: TOrigin) -> ValidateResult[Any, Literal[ResultFlag.ERROR]]: ...

    @overload
    def validate(self, input_: _T) -> ValidateResult[_T, Literal[ResultFlag.VALID]]: ...

    @overload
    def validate(
        self, input_: TOrigin, default: TDefault
    ) -> ValidateResult[TDefault, Literal[ResultFlag.DEFAULT]]: ...

    @overload
    def validate(self, input_: _T, default: Any) -> ValidateResult[_T, Literal[ResultFlag.VALID]]: ...

    def validate(self, input_: _T, default: Union[TDefault, Empty] = Empty) -> ValidateResult[_T | TDefault, ResultFlag]:  # type: ignore
        """
        对传入的值进行反向验证，返回可能的匹配与转化结果。

        若传入默认值，验证失败会返回默认值
        """
        try:
            res = self.base.match(input_)
        except MatchFailed:
            return ValidateResult(value=input_, flag=ResultFlag.VALID)
        else:  # pragma: no cover
            for i in self.base.validators + self.validators:
                if not i(res):
                    return ValidateResult(value=input_, flag=ResultFlag.VALID)
            if default is Empty:
                return ValidateResult(
                    error=MatchFailed(
                        lang.require("nepattern", "content_error").format(target=input_, expected=self._repr)
                    ),
                    flag=ResultFlag.ERROR,
                )
            if TYPE_CHECKING:
                default = cast(TDefault, default)
            return ValidateResult(default, flag=ResultFlag.DEFAULT)

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, AntiPattern) and self.base == other.base


TInput = TypeVar("TInput")


class CustomMatchPattern(BasePattern[TOrigin, TInput, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(
        self,
        origin: type[TOrigin],
        func: Callable[[BasePattern, TInput], TOrigin | None],
        alias: str | None = None,
    ):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=origin, alias=alias or func.__name__)
        self.__func__ = func
        self.match = func.__get__(self)  # type: ignore

    def __calc_eq__(self, other):  # pragma: no cover
        return isinstance(other, CustomMatchPattern) and self.__func__ == other.__func__


NONE = CustomMatchPattern(type(None), lambda _, __: None, alias="none")  # pragma: no cover


@final
class AnyPattern(BasePattern[Any, Any, Literal[MatchMode.KEEP]]):
    def __init__(self):
        super().__init__(mode=MatchMode.KEEP, origin=Any, alias="any")

    def match(self, input_: Any) -> Any:  # pragma: no cover
        return input_

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is AnyPattern


ANY = AnyPattern()
"""匹配任意内容的表达式"""


@final
class AnyStrPattern(BasePattern[str, Any, Literal[MatchMode.KEEP]]):
    def __init__(self):
        super().__init__(mode=MatchMode.KEEP, origin=str, alias="any_str")

    def match(self, input_: Any) -> str:
        return str(input_)

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is AnyStrPattern


AnyString = AnyStrPattern()
"""匹配任意内容并转为字符串的表达式"""


@final
class StrPattern(BasePattern[str, Union[str, bytes, bytearray], Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(
            mode=MatchMode.TYPE_CONVERT, origin=str, accepts=Union[str, bytes, bytearray], alias="str"
        )

    def match(self, input_: Any) -> str:
        if isinstance(input_, str):
            return input_.value if isinstance(input_, Enum) else input_
        elif isinstance(input_, (bytes, bytearray)):
            return input_.decode()
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected="str | bytes | bytearray"
            )
        )

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is StrPattern


STRING = StrPattern()


@final
class BytesPattern(BasePattern[bytes, Union[str, bytes, bytearray], Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(
            mode=MatchMode.TYPE_CONVERT, origin=bytes, accepts=Union[str, bytes, bytearray], alias="bytes"
        )

    def match(self, input_: Any) -> bytes:
        if isinstance(input_, bytes):
            return input_
        elif isinstance(input_, bytearray):
            return bytes(input_)
        elif isinstance(input_, str):
            return input_.encode()
        raise MatchFailed(
            lang.require("nepattern", "type_error").format(
                type=input_.__class__, target=input_, expected="bytes | str"
            )
        )

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is BytesPattern


BYTES = BytesPattern()


@final
class IntPattern(BasePattern[int, Any, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=int, alias="int")

    def match(self, input_: Any) -> int:
        if isinstance(input_, int) and input_ is not True and input_ is not False:
            return input_
        if isinstance(input_, (str, bytes, bytearray)) and len(input_) > 4300:  # pragma: no cover
            raise ValueError("int too large to convert")
        try:
            return int(input_)
        except (ValueError, TypeError, OverflowError) as e:
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected="int")
            ) from e

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is IntPattern


INTEGER = IntPattern()
"""整形数表达式，只能接受整数样式的量"""


@final
class FloatPattern(BasePattern[float, Any, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=float, alias="float")

    def match(self, input_: Any) -> float:
        if isinstance(input_, float):
            return input_

        try:
            return float(input_)
        except (TypeError, ValueError) as e:
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected="float")
            ) from e

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is FloatPattern


FLOAT = FloatPattern()
"""浮点数表达式"""


@final
class NumberPattern(BasePattern[Union[int, float], Any, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=Union[int, float], alias="number")  # type: ignore

    def match(self, input_: Any) -> int | float:
        if isinstance(input_, (float, int)):
            return input_
        try:
            res = float(input_)
            return int(res) if res.is_integer() else res
        except (ValueError, TypeError) as e:
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected="int | float")
            ) from e

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is NumberPattern


NUMBER = NumberPattern()
"""一般数表达式，既可以浮点数也可以整数 """


@final
class BoolPattern(BasePattern[bool, Union[str, bytes, bool], Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=bool, alias="bool")

    def match(self, input_: Any) -> bool:
        if input_ is True or input_ is False:
            return input_
        if isinstance(input_, bytes):  # pragma: no cover
            input_ = input_.decode()
        if isinstance(input_, str):
            input_ = input_.lower()
        if input_ == "true":
            return True
        if input_ == "false":
            return False
        raise MatchFailed(lang.require("nepattern", "content_error").format(target=input_, expected="bool"))

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is BoolPattern


BOOLEAN = BoolPattern()
"""布尔表达式，只能接受true或false样式的量"""


@final
class WideBoolPattern(BasePattern[bool, Any, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=bool, alias="bool")

    BOOL_FALSE = {0, "0", "off", "f", "false", "n", "no"}
    BOOL_TRUE = {1, "1", "on", "t", "true", "y", "yes"}

    def match(self, input_: Any) -> bool:
        if input_ is True or input_ is False:
            return input_
        if isinstance(input_, bytes):  # pragma: no cover
            input_ = input_.decode()
        if isinstance(input_, str):
            input_ = input_.lower()
        try:
            if input_ in self.BOOL_TRUE:
                return True
            if input_ in self.BOOL_FALSE:
                return False
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected="bool")
            )
        except (ValueError, TypeError) as e:
            raise MatchFailed(
                lang.require("nepattern", "type_error").format(
                    type=input_.__class__, target=input_, expected="bool"
                )
            ) from e

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is BoolPattern


WIDE_BOOLEAN = WideBoolPattern()
"""宽松布尔表达式，可以接受更多的布尔样式的量"""

LIST = BasePattern(r"(\[.+?\])", MatchMode.REGEX_CONVERT, list, alias="list")
TUPLE = BasePattern(r"(\(.+?\))", MatchMode.REGEX_CONVERT, tuple, alias="tuple")
SET = BasePattern(r"(\{.+?\})", MatchMode.REGEX_CONVERT, set, alias="set")
DICT = BasePattern(r"(\{.+?\})", MatchMode.REGEX_CONVERT, dict, alias="dict")

EMAIL = BasePattern(r"(?:[\w\.+-]+)@(?:[\w\.-]+)\.(?:[\w\.-]+)", MatchMode.REGEX_MATCH, alias="email")
"""匹配邮箱地址的表达式"""

IP = BasePattern(
    r"(?:(?:[01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5])\.){3}(?:[01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5]):?(?:\d+)?",
    MatchMode.REGEX_MATCH,
    alias="ip",
)
"""匹配Ip地址的表达式"""

URL = BasePattern(
    r"(?:\w+://)?[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(?:\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})+(?::[0-9]{1,5})?[-a-zA-Z0-9()@:%_\\\+\.~#?&//=]*",
    MatchMode.REGEX_MATCH,
    alias="url",
)
"""匹配网页链接的表达式"""


@final
class HexPattern(BasePattern[int, str, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=int, alias="hex", accepts=str)

    def match(self, input_: str) -> int:
        if not isinstance(input_, str):
            raise MatchFailed(
                lang.require("nepattern", "type_error").format(
                    type=input_.__class__, target=input_, expected="str"
                )
            )
        try:
            return int(input_, 16)
        except ValueError as e:
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected="hex")
            ) from e

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is HexPattern


HEX = HexPattern()
"""匹配16进制数的表达式"""

HEX_COLOR = BasePattern(r"(#[0-9a-fA-F]{6})", MatchMode.REGEX_CONVERT, str, lambda _, x: x[1][1:], "color")
"""匹配16进制颜色代码的表达式"""


@final
class DateTimePattern(BasePattern[datetime, Union[str, int, float], Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(
            mode=MatchMode.TYPE_CONVERT, origin=datetime, alias="datetime", accepts=Union[str, int, float]
        )

    def match(self, input_: Union[str, int, float]) -> datetime:
        if isinstance(input_, (int, float)):
            return datetime.fromtimestamp(input_)
        if not isinstance(input_, str):
            raise MatchFailed(
                lang.require("nepattern", "type_error").format(
                    type=input_.__class__, target=input_, expected="str | int | float"
                )
            )
        return DateParser.parse(input_)

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is DateTimePattern


DATETIME = DateTimePattern()
"""匹配时间的表达式"""


@final
class PathPattern(BasePattern[Path, Any, Literal[MatchMode.TYPE_CONVERT]]):
    def __init__(self):
        super().__init__(mode=MatchMode.TYPE_CONVERT, origin=Path, alias="path")

    def match(self, input_: Any) -> Path:
        if isinstance(input_, Path):
            return input_

        try:
            return Path(input_)
        except (ValueError, TypeError) as e:
            raise MatchFailed(
                lang.require("nepattern", "content_error").format(target=input_, expected="PathLike")
            ) from e

    def __calc_eq__(self, other):  # pragma: no cover
        return other.__class__ is PathPattern


PATH = PathPattern()

PathFile = BasePattern(
    mode=MatchMode.TYPE_CONVERT,
    origin=bytes,
    accepts=Path,
    previous=PATH,
    alias="filedata",
    converter=lambda _, x: x.read_bytes() if x.exists() and x.is_file() else None,
)


def combine(
    current: BasePattern[TOrigin, TInput, TMM],
    previous: BasePattern[TInput, Any, Any] | None = None,
    alias: str | None = None,
    validators: list[Callable[[TOrigin], bool]] | None = None,
) -> BasePattern[TOrigin, TInput, TMM]:
    _new = current.copy()
    if previous:
        _match = _new.match

        def match(self, input_):
            return _match(previous.match(input_))

        _new.match = match.__get__(_new)
    if alias:
        _new.alias = alias
        _new.refresh()
    if validators:
        _new.validators = validators
    return _new


DelimiterInt = combine(
    INTEGER,
    BasePattern(mode=MatchMode.VALUE_OPERATE, origin=str, converter=lambda _, x: x.replace(",", "_")),
    "DelimInt",
)
